#include <iostream>
#include <sstream>
#include <stack>
#include <vector>
#include <optional>
#include <set>
#include <map>
#include <stdexcept>
#include <algorithm>
#include <bitset>
#include <cmath>
#include "validation.h"
#define K 15

using namespace std;

namespace binary_speakers {
    using TruthTable = std::bitset<1 << K>;
    TruthTable global_tt;

    std::set<std::string> CONNECTIVES;
    std::map<std::string, int> ARITIES;
    std::map<std::string, int> PROJS;
    std::map<std::string, TruthTable> INTERPRETATIONS;
    std::set<char> NON_CONNECTIVE_CHARS = {'x',')','(',','};

    struct Fmla {
        bool is_prop = false;
        std::string head = "";
        std::vector<Fmla> components;
        Fmla() {}
        Fmla(const std::string& _head) : is_prop {true}, head{_head} {} 
        Fmla(const std::string& _head, const std::vector<Fmla>& _components) 
            : is_prop{false}, head{_head}, components{_components} {}
        std::set<std::string> vars() const {
            if (is_prop) return {head};
            std::set<std::string> _vars = {};
            for (const auto& c : components) {
                auto cvars = c.vars();
                _vars.insert(cvars.begin(), cvars.end());
            }
            return _vars;
        }
        bool operator==(const Fmla& other) {
            if (this->is_prop) return (other.is_prop and this->head == other.head);
            if (other.is_prop) return false;
            if (this->head != other.head) return false;
            if (this->components.size() != other.components.size()) return false;
            bool eq = true;
            for (size_t i {0}; i < this->components.size(); ++i)
                eq &= (this->components[i] == other.components[i]);
            return eq;
        }
        bool operator!=(const Fmla& other) { return not (*this == other); }
        std::string print() const {
            if (is_prop) return head;
            std::string r = head + "(";
            for (size_t i = 0; i < components.size(); ++i) {
                r += components[i].print();
                if (i != components.size() - 1) r += ",";
            }
            r += ")";
            return r;   
        }
    };

    Fmla parse(const std::string& expression, int target_arity) {
        std::stack<std::string> connectives;
        std::stack<Fmla> components;

        size_t i = 0;

        while (i < expression.size()) {
            std::string connective = "";
            while (i < expression.size() and NON_CONNECTIVE_CHARS.find(expression[i]) == NON_CONNECTIVE_CHARS.end())
                connective += expression[i++]; 
            if(connective != "" and CONNECTIVES.find(connective) != CONNECTIVES.end()) {
                connectives.push(connective);
                if (i >= expression.size())
                    throw std::invalid_argument("parse error at position " + std::to_string(i));
                if (i < expression.size() and expression[i] != '(')
                    throw std::invalid_argument("parse error at position " + std::to_string(i));
                if (connective.rfind("pi", 0) == 0 and i < expression.size() - 1 and expression[i+1] != 'x') 
                    throw std::invalid_argument("parse error at position " + std::to_string(i));
                ++i;
                if (connective.rfind("pi", 0) == 0 and i < expression.size() and expression[i] == 'x') {
                    Fmla prop {"x"};
                    components.push(prop);
                    ++i;
                    if (i >= expression.size() or (i < expression.size() and expression[i] != ')'))
                        throw std::invalid_argument("parse error at position " + std::to_string(i));
                }
            } else if (i < expression.size() and (expression[i] == ',' or expression[i] == '(')) {
                i++;
                if (i < expression.size() and (expression[i] == ',' or expression[i] == '(' or expression[i] == ')'))
                    throw std::invalid_argument("parse error at position " + std::to_string(i));
            } else if (i < expression.size() and expression[i] == ')') {
                // time to build a new component!
                if (connectives.empty())
                    throw std::invalid_argument("parse error at position " + std::to_string(i));
                string connective = connectives.top();
                connectives.pop();
                std::vector<Fmla> components_current;
                while (not components.empty() and ((int) components_current.size() < ARITIES[connective])) {
                    components_current.insert(components_current.begin(), components.top());
                    components.pop();
                }
                if ((int) components_current.size() != ARITIES[connective])
                    throw std::invalid_argument("parse error at position " + std::to_string(i) + " - arity mismatch");
                Fmla new_component {{connective}, components_current};
                components.push(new_component);
                i++;
                if (i < expression.size() and expression[i] != ',' and expression[i] != ')')
                    throw std::invalid_argument("parse error at position " + std::to_string(i));
            }
            else {
                if (i < expression.size())
                    throw std::invalid_argument("unknown char at position " + std::to_string(i));
            }
        }

        if ((not connectives.empty()) or (not (components.size() == 1)))
            throw std::invalid_argument("parse error");
        else
            return components.top();
    }

    int get_ith_bin_digit(int n, int i) {
        return (n >> i) & 1;
    }

    int evaluate(const Fmla& fm, int value_of_x, int k) {
        if (fm.is_prop) {
            return value_of_x;
        } else if (fm.head.rfind("pi", 0) == 0){
            auto proj_index = PROJS.find(fm.head);
            if (proj_index == PROJS.end())
                throw std::invalid_argument("unknown projection");
            //cerr << fm.head << " " << (*proj_index).second << "[" << value_of_x << "] = " << get_ith_bin_digit(value_of_x, k - (*proj_index).second)  << std::endl;
            return get_ith_bin_digit(value_of_x, k - (*proj_index).second); 
        }
        else {
            int input_to_interp = 0;
            auto interp = INTERPRETATIONS.find(fm.head);
            if (interp == INTERPRETATIONS.end())
                throw std::invalid_argument("unknown connective to interpret");
            for (size_t i = 0; i < fm.components.size(); ++i)
                input_to_interp += (evaluate(fm.components[i], value_of_x, k)*(1 << (3-i-1)));
            //cerr << "s [" << input_to_interp << "] = " << (*interp).second[input_to_interp] << std::endl;
            return (*interp).second[input_to_interp];
        }
    }

    bool is_same_function(const TruthTable& target_function, int target_arity, const Fmla& fm){
        //cerr << "arity is " << target_arity << std::endl;
        //cerr << fm.print() << std::endl;
        for (long long i = 0; i < (1<<target_arity); i++) {
            //cerr << "f[" << i << "] = " << target_function[i] << std::endl;
            //cerr << "exp[" << i << "] = " << evaluate(fm, i, target_arity) << std::endl;
            if (target_function[i] != evaluate(fm, i, target_arity))
                return false;
        } 
        return true;
    }

    /**
     * @param k the arity of the function we want to test if the expression match 
     */
    void make_base_functions(int k) {
        CONNECTIVES.clear();
        ARITIES.clear();
        INTERPRETATIONS.clear();
        PROJS.clear();
        
        CONNECTIVES.insert("s");
        ARITIES["s"] = 3;
        INTERPRETATIONS["s"] = {0b11001010};

        for (int j = 1; j <= k; ++j) {
            TruthTable projkj;
            long long maxval = 1 << k;
            for (long long i = 0; i < maxval; ++i) {
                projkj[i] = get_ith_bin_digit(i, k-j);//((i >> (k-j)) & 1); 
            }
            std::string proj_name = "pi" + std::to_string(j);
            CONNECTIVES.insert(proj_name);
            ARITIES[proj_name] = 1;
            PROJS[proj_name] = j;
            INTERPRETATIONS[proj_name] = projkj;
        }
    }

    // O(k)
    int to_dec(const vector<bool>& in) {
        int r = 0;
        for (size_t i {0}; i < in.size(); i++)
            r += in[i] * (1 << (in.size() - i - 1));
        return r;
    }

    TruthTable make_tt(const vector<vector<bool>>& inputs, int k) {
        TruthTable tt;
        for (auto i {0}; i < (1 << k); ++i) 
            tt[i] = 1;
        for (const auto& in : inputs) 
            tt[to_dec(in)] = 0;
        return tt;
    }


    bool process_input(istream& stream, std::function<bool(const pair<TruthTable, int>&)> callback) {
        int N;
        stream >> N;
        while (N--) {
            int k, z;
            stream >> k >> z; 

            vector<vector<bool>> zero_inputs;
            for (int i = 0; i < z; ++i) {
                vector<bool> input;
                for (int j = 0; j < k; ++j) {
                    int v;
                    stream >> v;
                    input.push_back(v);
                }
                zero_inputs.push_back(input);
            }

            auto tt = make_tt(zero_inputs, k);
            if (not callback({tt,k}))
                return false;
        }
        return true;
    }

    vector<pair<TruthTable, int>> parse_input(istream& stream, std::function<void(const pair<TruthTable, int>&)> callback) {
        vector<pair<TruthTable, int>> result;

        int N;

        stream >> N;

        while (N--) {
            int k, z;
            stream >> k >> z; 

            vector<vector<bool>> zero_inputs;
            for (int i = 0; i < z; ++i) {
                vector<bool> input;
                for (int j = 0; j < k; ++j) {
                    int v;
                    stream >> v;
                    input.push_back(v);
                }
                zero_inputs.push_back(input);
            }

            auto tt = make_tt(zero_inputs, k);
            result.push_back({tt, k});
            callback({tt,k});
        }

        return result;
    }

    pair<string,string> parse_output_line(const string& line) {
        if (line == "no")
            return {"no",""};  
        else {
            unsigned int separator_pos = line.find(","); 
            if (separator_pos == string::npos)
                throw std::logic_error("output parsing error, no comma found");
            auto yn = line.substr(0, separator_pos);
            if (yn != "yes")
                throw std::logic_error("output parsing error, expected yes, but got " + yn);
            auto expr = line.substr(separator_pos + 1, line.length());
            return {yn, expr};
        }
    }

    //vector<pair<string,string>> parse_output(const string& output) {
    vector<pair<string,string>> parse_output(istream& stream) {
        vector<pair<string,string>> result;
        string line;
        while (!stream.eof()) {
            stream >> line;
            if (line == "no")
                result.push_back({"no",""});  
            else {
                unsigned int separator_pos = line.find(","); 
                if (separator_pos == string::npos)
                    throw std::logic_error("output parsing error, no comma found");
                auto yn = line.substr(0, separator_pos);
                if (yn != "yes")
                    throw std::logic_error("output parsing error, expected yes, but got " + yn);
                auto expr = line.substr(separator_pos + 1, line.length());
                result.push_back({yn, expr});
            }
        }
        return result;
    }

    bool check(const TruthTable& target_function, int target_arity, const string& provided_expression) {
        make_base_functions(target_arity);
        auto fmla = parse(provided_expression, target_arity);
        return is_same_function(target_function, target_arity, fmla);
    }

    // O(1)
    bool check_expressible(const TruthTable& tt, int k) {
        return !tt.test(0) and tt.test((1 << k) - 1);
    }

    bool verify_case(const TruthTable& tt, int k, const pair<string,string>& parsed_oficial_answer, const pair<string,string>& parsed_contestant_answer) {
        if (!check_expressible(tt, k)) {
            if (parsed_contestant_answer.first != "no") 
                throw std::logic_error("wrong answer 1");
            else
                return true;
        } else {
            if (parsed_contestant_answer.first != "yes") 
                throw std::logic_error("wrong answer 2");
            else {
                auto contestant_expr = parsed_contestant_answer.second;
                if (not check(tt, k, contestant_expr)) 
                    throw std::logic_error("wrong answer 3 " + contestant_expr);
                else
                    return true;
            } 
        }
        return false;
    }

    bool verify(istream& input, istream& contestant_output, istream& jury_output) {
        auto parsed_contestant_output = parse_output(contestant_output);
        auto parsed_jury_output = parse_output(jury_output);
        if (parsed_contestant_output == parsed_jury_output)
            return true;
        auto parsed_input = parse_input(input, [&](const pair<TruthTable, int>&){});
        if (parsed_input.size() != parsed_contestant_output.size())
            return false;
        for (size_t i = 0; i < parsed_jury_output.size(); ++i) {
            if (parsed_jury_output[i].first == "no") {
                if (parsed_contestant_output[i].first != "no") 
                    return false;
            } else {
                if (parsed_contestant_output[i].first != "yes") return false;
                else {
                    auto contestant_expr = parsed_contestant_output[i].second;
                    auto input_truth_table = parsed_input[i].first;
                    auto input_arity = parsed_input[i].second;
                    if (not check(input_truth_table, input_arity, contestant_expr)) 
                        return false;
                } 
            }
        }
        return true;
    }
}
