#include <iostream>
#include <bitset>
#include <string>
#include <vector>
#include <cmath>
#include <optional>
#define K 15

using namespace std;

// O(1)
string make_proj_expr(int k, int j) {
    return "pi" + to_string(j) + "(x)";
}

// O(1)
string make_s_expr(string f1, string f2, string f3) {
    return "s(" + f1 + "," + f2 + "," + f3 + ")";
}

using TruthTable = bitset<1 << K>;

TruthTable global_tt;

int to_dec(const vector<bool>& in) {
    int r = 0;
    for (size_t i {0}; i < in.size(); i++)
        r += in[i] * (1 << (in.size() - i - 1));
    return r;
}

// O(2^k)
void make_tt(const vector<vector<bool>>& inputs, int k) {
    for (auto i {0}; i < (1 << k); ++i) 
        global_tt[i] = 1;
    for (const auto& in : inputs) 
        global_tt[to_dec(in)] = 0;
}

// O(1)
bool check_expressible(const TruthTable& tt, int k) {
    return !tt.test(0) and tt.test((1 << k) - 1);
}

// O(k)
optional<int> first_pos_with(const vector<bool> suffix_fixed, bool val, int k) {
    for (auto i{0}; i < suffix_fixed.size(); ++i)
        if (suffix_fixed[i] == val) return ((i+1) + (k - suffix_fixed.size()));
    return nullopt; // expected to never happen!
}

// O(k)
int check_in_induced_table(const TruthTable& tt, const vector<bool> fixed_suffix, int k, int check_pos) {
    int induced_dim = k - fixed_suffix.size();
    int pos_in_original_table = (check_pos << fixed_suffix.size()) + to_dec(fixed_suffix);
    return tt[pos_in_original_table];
}


// O(1)
int get_ith_bin_digit(int n, int i) {
    return (n >> i) & 1;
}

// O(2^k)
optional<int> is_projection_induced_table(const TruthTable& tt, const vector<bool> fixed_suffix, int k) {
    int induced_dim = k - fixed_suffix.size();
    int j = 1;
    while (j <= induced_dim) {
        int i = 0;
        while (i < (1 << induced_dim) and (check_in_induced_table(tt, fixed_suffix, k, i) == get_ith_bin_digit(i, induced_dim - j)))
            i++;
        if (i >= (1 << induced_dim))
            break;
        ++j;
    }
    if (j <= induced_dim)
        return j;
    return nullopt;
}

// O(2^k)
optional<int> is_in_fixed_suffix(const TruthTable& tt, const vector<bool> fixed_suffix, int k) {
    int induced_dim = k - fixed_suffix.size();
    int val = check_in_induced_table(tt, fixed_suffix, k, 0);
    bool same = true;
    int i = 1;
    while (i < pow(2, induced_dim) and same)
        same = (check_in_induced_table(tt, fixed_suffix, k, i++) == val);
    if (not same) return nullopt;
    else {
        int j = 0;
        for (; j < fixed_suffix.size() and fixed_suffix[j] != val; ++j);
        if (j < fixed_suffix.size()) return j + induced_dim;
        else return nullopt;
    }
}

string make_expr(const TruthTable& tt, vector<bool> suffix_fixed, int k) {
    // O(2^k)
    if (auto pos = is_projection_induced_table(tt, suffix_fixed, k); pos)
        return make_proj_expr(k, *pos);
    if (auto pos = is_in_fixed_suffix(tt, suffix_fixed, k); pos)
        return make_proj_expr(k, *pos + 1);
    if (suffix_fixed.size() == k) {
        bool funval = tt[to_dec(suffix_fixed)];
        int pos = *first_pos_with(suffix_fixed, funval, k); // note that the preservation condition guarantees I can do this!
        return make_proj_expr(k, pos);
    }

    // O(1)
    vector<bool> suffix_plus_one = suffix_fixed;
    suffix_plus_one.insert(suffix_plus_one.begin(), 1);
    vector<bool> suffix_plus_zero = suffix_fixed;
    suffix_plus_zero.insert(suffix_plus_zero.begin(), 0);

    int current_fixing_position = suffix_fixed.size() + 1;

    // O(k)
    auto Ftest = make_proj_expr(k, k - current_fixing_position + 1);

    // F(N-1)
    auto Fone = make_expr(tt, suffix_plus_one, k);

    // F(N-1)
    auto Fzero = make_expr(tt, suffix_plus_zero, k);

    // O(1)
    return make_s_expr(Ftest, Fone, Fzero);
}

int main() {

    int N;

    cin >> N;

    while (N--) {
        int k, z;
        cin >> k >> z; 

        vector<vector<bool>> zero_inputs;
        for (int i = 0; i < z; ++i) {
            vector<bool> input;
            for (int j = 0; j < k; ++j) {
                int v;
                cin >> v;
                input.push_back(v);
            }
            zero_inputs.push_back(input);
        }

        make_tt(zero_inputs, k);
        if (check_expressible(global_tt, k)) {
            cout << "yes,";
            cout << make_expr(global_tt, {}, k) << endl;
        } else {
            cout << "no" << endl;
        }

    }

    return 0;

}
